#####################################################
## This code was used to analyze the example in the paper:
## Hlavka, Huskova, Kirch, and Meintanis (2015): Fourier–Type Tests Involving
## Martingale Difference Processes, Econometric Reviews, ahead of print, DOI:
## 10.1080/07474938.2014.977074.

## The functions for calculating the test statistics (and p-values) are
## implemented in C and shared libraries must be created using the command "R
## CMD SHLIB *.c" (in Linux). This command may look different on other
## operating systems and it might be necessary to modify also the dyn.load()
## functions for loading the resulting shared libraries.

## Other necessary files are "sap500yahoo.csv" (data set), mdh0w1.c,
## mdhs0w1.c, mdh30w1.c, and mdh30w1c.c (source codes for calculating p-values
## and change-point estimator in C). Note that the C functions must be
## compiled before running the code (it may be necessary to modify the
## dyn.load() function on other operating systems than Linux.  See
## www.r-project.org for more information on shared libraries.

## The p-values are calculated by wild bootstrap - this means that the
## p-values may be each time a bit different.

#####################################################
## Z. Hlavka, 20.1.2015, hlavka@karlin.mff.cuni.cz ##
#####################################################

## Read the data set S&P 500 (downloade from YAHOO)
sap=read.csv("sap500yahoo.csv")

sap=sap[nrow(sap):1,]

year=as.numeric(substr(as.character(sap$Date),1,4))

sap.mdh=sap[(year<=1997)&(year>=1990),"Close"] 
date.mdh=sap[(year<=1997)&(year>=1990),"Date"]
year.mdh=year[(year<=1997)&(year>=1990)]

mdh.data=data.frame(sap=sap.mdh,date.mdh,year.mdh)

# Hypothesis H1 (non-sequential):
# we define function for calculating test statistics and p-value (wild bootstrap) for the first hypothesis 
mdh.wb1 <- function (y,m=2,a=1,B=1000,...) { 
  mdh=.C("mdh0w1wb",y=as.double(y),n=as.integer(length(y)),m=as.integer(m),a=as.double(a),b=as.integer(B),eta=0.5+(2*(runif(length(y)*B)>0.5+1/(2*sqrt(5)))-1)*sqrt(5)/2,t=double(2))$t
  list(mdh0=mdh[1],pval=mdh[2])
}
dyn.load("./mdh0w1.so") # the shared library was created as "R CMD SHLIB mdh0w1.c"

# Hypothesis H2 (sequential, change from MDH to non-MDH):
# we define function for calculating test statistics and p-value (wild bootstrap) for the second hypothesis 
mdh.wb2 <- function (y,m=2,a=1,g=0.5,B=1000,...) { 
  mdh=.C("mdhs0w1wb",y=as.double(y),n=as.integer(length(y)),m=as.integer(m),a=as.double(a),g=as.double(g),b=as.integer(B),eta=0.5+(2*(runif(length(y)*B)>0.5+1/(2*sqrt(5)))-1)*sqrt(5)/2,t=double(2))$t
  list(mdh0=mdh[1],pval=mdh[2])
}
dyn.load("./mdhs0w1.so")

# Hypothesis H3 (sequential, generalization of H2):
# we define function for calculating test statistics and p-value (wild bootstrap) for the third hypothesis 
mdh.wb3 <- function (y,m=2,a=1,g=0.5,B=1000,...) { 
  mdh=.C("mdh30w1wb",y=as.double(y),n=as.integer(length(y)),m=as.integer(m),a=as.double(a),g=as.double(g),b=as.integer(B),eta=0.5+(2*(runif(length(y)*B)>0.5+1/(2*sqrt(5)))-1)*sqrt(5)/2,t=double(2))$t
  list(mdh0=mdh[1],pval=mdh[2])
}
dyn.load("./mdh30w1.so")

# Define function for "changepoint estimation"
mdh.wb3c <- function (y,m=2,a=1,g=0.5,...) { 
  mdh=.C("mdh30w1c",y=as.double(y),n=as.integer(length(y)),m=as.integer(m),a=as.double(a),g=as.double(g),k0=integer(1),t=double(2))
  list(mdht=mdh$t,k0=mdh$k0) 
}
dyn.load("./mdh30w1c.so")

## calculate log returns
y = 100*log(sap.mdh[-1]/sap.mdh[-length(sap.mdh)]) 

mdh.data=mdh.data[-1,]
mdh.data$y=y

## graphical output
#pdf("sap500cp.pdf",height=4,width=8)
plot(y,pch=20,cex=0.5,xlab=expression(k),ylab="100 x log returns",main="S&P 500")
abline(v=1013,lty=2)
abline(v=1250)
#dev.off()

y.1=y[year.mdh[-1]<=1993]
y.2=y[year.mdh[-1]>1993]

## first (non-sequential) test on both parts of the data set:
mdh.wb1(y.1,m=1,a=1,B=1000)$pval
mdh.wb1(y.2,m=1,a=1,B=1000)$pval
#[1] 0.476
#[1] 0.005


## second (sequential) test:
mdh.wb2(y,m=1,a=1,g=0.5,B=1000)$pval
#[1] 0.003
mdh.wb2(y,m=2,a=1,g=0.5,B=1000)$pval
#[1] 0.003
mdh.wb2(y,m=3,a=1,g=0.5,B=1000)$pval
#[1]

## third (sequential) test:
mdh.wb3(y,m=1,a=1,g=0.5,B=1000)$pval
#[1] 0.264

## change-point estimator:
mdh.wb3c(y,m=1,a=1,g=0.5,B=1000)
#$mdht
#[1] 2.419472 0.000000
#$k0
#[1] 1249
# mdh.data$date[1249]
# [1] 1994-12-08

## investigate the estimated changepoint using the non-sequential test:
mdh.wb1(y[1:1248],m=1,a=1,B=1000)$pval
mdh.wb1(y[1249:length(y)],m=1,a=1,B=1000)$pval
#[1] 0.611
#[1] 0

## investigate the estimated changepoint using the sequential test of H2:
mdh.wb2(y[1:1248],m=1,a=1,g=0.5,B=1000)$pval
#[1] 0.502

